Class: Pione::System::FileCache::FileCacheMethod
- Inherits:
-
Object
- Object
- Pione::System::FileCache::FileCacheMethod
- Defined in:
- lib/pione/system/file-cache.rb
Overview
FileCache is an interface class of cache methods.
Direct Known Subclasses
Instance Method Summary collapse
-
#cached?(location) ⇒ Boolean
Return true if the location is cached.
-
#clear ⇒ void
Clear cache.
-
#get(location) ⇒ BasicLocalLocation
Get the cache path of the location.
-
#put(src, location) ⇒ void
Put and cache the source data at the location.
-
#sync(old_location, new_location) ⇒ void
Synchronize cache of old location with new location.
Instance Method Details
#cached?(location) ⇒ Boolean
Return true if the location is cached.
115 116 117 |
# File 'lib/pione/system/file-cache.rb', line 115 def cached?(location) raise NotImplementedError end |
#clear ⇒ void
This method returns an undefined value.
Clear cache.
122 123 124 |
# File 'lib/pione/system/file-cache.rb', line 122 def clear raise NotImplementedError end |
#get(location) ⇒ BasicLocalLocation
Get the cache path of the location.
85 86 87 |
# File 'lib/pione/system/file-cache.rb', line 85 def get(location) raise NotImplementedError end |
#put(src, location) ⇒ void
This method returns an undefined value.
Put and cache the source data at the location.
96 97 98 |
# File 'lib/pione/system/file-cache.rb', line 96 def put(src, location) raise NotImplementedError end |
#sync(old_location, new_location) ⇒ void
This method returns an undefined value.
Synchronize cache of old location with new location.
107 108 109 |
# File 'lib/pione/system/file-cache.rb', line 107 def sync(old_location, new_location) raise NotImplementedError end |